/*
 * menu.h: Web video plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#ifndef __WEBVIDEO_MENU_H
#define __WEBVIDEO_MENU_H

#include <vdr/osdbase.h>
#include <vdr/menuitems.h>
#include <vdr/menu.h>
#include <libxml/parser.h>
#include "download.h"

// --- cLinkBase -----------------------------------------------------------

class cLinkBase {
public:
  virtual ~cLinkBase() {}; // avoids "virtual functions but
			   // non-virtual destructor" warning

  virtual char *GetURL() = 0;
};

// --- cQueryData ----------------------------------------------------------

class cQueryData {
public:
  virtual ~cQueryData() {}; // avoids "virtual functions but
			    // non-virtual destructor" warning

  virtual char *GetQueryFragment() = 0;
};

// --- cSimpleLink ---------------------------------------------------------

class cSimpleLink : public cLinkBase {
private:
  char *ref;
public:
  cSimpleLink(const char *ref);
  ~cSimpleLink();

  virtual char *GetURL();
};

// --- cTextFieldData ------------------------------------------------------

class cTextFieldData : public cQueryData {
private:
  char *id;
  char *valuebuffer;
  int valuebufferlength;
public:
  cTextFieldData(int Length, const char *ID);
  ~cTextFieldData();

  virtual char *GetQueryFragment();
  char *GetValue();
  int GetLength();
};

// --- cItemListData -------------------------------------------------------

class cItemListData : public cQueryData {
private:
  char *id;
  int value;
  int numstrings;
  char **strings;
  char **stringvalues;
public:
  cItemListData(char **Strings, char **StringValues, int NumStrings, const char *ID);
  ~cItemListData();

  virtual char *GetQueryFragment();
  char **GetStrings();
  char **GetStringValues();
  int GetNumStrings();
  int *GetValuePtr();
};

// --- cSubmissionButtonData -----------------------------------------------

class cSubmissionButtonData : public cLinkBase {
private:
  cVector<cQueryData *> *queryObjects;
  char *querybase;
public:
  cSubmissionButtonData(const char *queryUrl, cVector<cQueryData *> *objects);
  ~cSubmissionButtonData();

  virtual char *GetURL();
};

// --- cXMLMenu --------------------------------------------------

class cXMLMenu : public cOsdMenu {
protected:
  virtual void Deserialize(xmlDocPtr doc, xmlNodePtr root);
  virtual bool CreateItemFromTag(xmlDocPtr doc, xmlNodePtr node) = 0;
public:
  cXMLMenu(const char *Title, int c0 = 0, int c1 = 0, 
           int c2 = 0, int c3 = 0, int c4 = 0);

  int Load(const char *xmlstr);
};

// --- cNavigationMenu -----------------------------------------------------

enum eLinkType { LT_REGULAR, LT_OBJECT, LT_STREAMOBJECT };

class cHistory;
class cHistoryObject;

class cNavigationMenu : public cXMLMenu {
private:
  // links[i] is the navigation link of the i:th item
  cVector<cLinkBase *> links;
  // objectLinks[i] is the media object link of the i:th item
  cVector<cLinkBase *> objectLinks;
  // queryObjects contains items that should be included in the menu's query
  cVector<cQueryData *> queryObjects;
protected:
  cHistory *history;
  cCommunicationThread *downloadthread;

  virtual bool CreateItemFromTag(xmlDocPtr doc, xmlNodePtr node);
  void AddLinkItem(cOsdItem *item, cLinkBase *ref, 
                   cLinkBase *objectref, cQueryData *queryobject);
  void NewLinkItem(xmlDocPtr doc, xmlNodePtr node);
  void NewTextField(xmlDocPtr doc, xmlNodePtr node);
  void NewItemList(xmlDocPtr doc, xmlNodePtr node);
  void NewTextArea(xmlDocPtr doc, xmlNodePtr node);
  void NewButton(xmlDocPtr doc, xmlNodePtr node);
  void NewTitle(xmlDocPtr doc, xmlNodePtr node);
public:
  cNavigationMenu(cHistory *History, cCommunicationThread *dlthread);
  virtual ~cNavigationMenu();

  virtual eOSState ProcessKey(eKeys Key);
  virtual eOSState Select(cLinkBase *link, eLinkType type);
  virtual void Clear(void);
  eOSState HistoryBack();
  eOSState HistoryForward();

  void Populate(cHistoryObject *page, const char *statusmsg=NULL);
  void UpdateHelp();
};

// --- cStatusScreen -------------------------------------------------------

class cStatusScreen : public cOsdMenu {
private:
  cVector<cFileDownloadSummary *> summaries;
  cCommunicationThread *downloadthread;
public:
  cStatusScreen(cCommunicationThread *thread);
  ~cStatusScreen();

  virtual eOSState ProcessKey(eKeys Key);
};

#endif // __WEBVIDEO_MENU_H
